/*
  Pinkit - Fun image viewer

  Copyright (C) 2012 Petteri Klemola <petteri@gmail.com>

  Pinkit is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 3 as
  published by the Free Software Foundation.

  Pinkit is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  02110-1301, USA.

  For more information about Pinkit visit http://23.fi/pinkit .

*/

import QtQuick 1.1
import com.nokia.meego 1.0
import QtMobility.systeminfo 1.1

Page {
    id: imgPage
    Component.onCompleted: theme.color

    function get_lines(string)
    {
        var limit = 17;
        var new_string = "<html>"
        if (string.length < limit)
        {
            new_string += string;
        }
        else
        {
            limit = 25;
            var line = "";
            var space = "";
            var words = string.split(" ");
            for (var i in words)
            {
                var tmp = line + space + words[i];
                if (tmp.length > limit)
                {
                    new_string += line;
                    new_string += "<br />"
                    line = words[i];
                    space = " ";
                }
                else
                {
                    line += space;
                    line += words[i];
                    space = " ";
                }
            }
            new_string += space;
            new_string += line;
        }
        return new_string + "</html>"
    }

    function get_font_size(str_len)
    {
        return (str_len < 17) ? 40 : 28;
    }

    function next_image()
    {
        if ((curModel.currentIndex + 1) < curModel.count)
            curModel.currentIndex++;
        else
            curModel.currentIndex = 0;
    }

    function previous_image()
    {
        if ((curModel.currentIndex - 1) >= 0)
            curModel.currentIndex--;
        else
            curModel.currentIndex = curModel.count -1;
    }

    ScreenSaver {
        id: imgScreenSaver
    }

    Rectangle {
        id: headerBackground
        color: "transparent"
        width: parent.width
        height: 900
        anchors.top: parent.top
        z: 25

        Text {
            id: titleText
            text: { imgPage.get_lines(curModel.currentItem.myData.title);}
            font.pointSize: { imgPage.get_font_size(curModel.currentItem.myData.title.length);}
            color: "white"
            anchors.top: headerBackground.top
            anchors.left: parent.left
            anchors.leftMargin: 10
        }
    }

    Rectangle {
        id: centerPage
        color: "transparent"
        width: parent.width
        height: parent.height
        property int pressedx: 0
        property int pressedy: 0

        MouseArea {
            anchors.fill: parent
            onPressed: {
                /* Stop slideshow if it is running */
                if (imgTimer.running)
                {
                    toolBack.visible = true;
                    toolMenu.visible = true;
                    imgTimer.running = false;
                    imgScreenSaver.setScreenSaverDelayed(false);
                }
                centerPage.pressedx = mouse.x;
                centerPage.pressedy = mouse.y;
            }
            onReleased: {
                var limit_right = mouse.x + 30;
                var limit_left = mouse.x - 30;
                if (centerPage.pressedx > limit_right)
                    next_image();
                else if (centerPage.pressedx < limit_left)
                    previous_image();
                else if (toolBack.visible && toolMenu.visible)
                    {
                        toolBack.visible = false;
                        toolMenu.visible = false;
                        titleText.visible = false;
                    }
                else if (!toolBack.visible && !toolMenu.visible)
                    {
                        toolBack.visible = true;
                        toolMenu.visible = true;
                        titleText.visible = true;
                    }
            }
        }

        BusyIndicator {
            id: busyScreen
            platformStyle: BusyIndicatorStyle { size: "large" }
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            visible: true
            running: true
        }
    }

    AnimatedImage {
        id: image
        width: parent.width
        height: parent.height
        fillMode: AnimatedImage.PreserveAspectFit
        smooth: true
        source: curModel.currentItem.myData.url
        onStatusChanged: {
            if (image.status == Image.Ready)
            {
                busyScreen.running = false;
                busyScreen.visible = false;
                image.visible = true;
            }
            else if (image.status == Image.Loading)
            {
                image.visible = false;
                busyScreen.running = true;
                busyScreen.visible = true;
            }
        }
        Timer {
            id: imgTimer
            interval: 10000
            running: false
            repeat: true
            onTriggered: { next_image(); }
        }
    }

    ToolIcon {
        id: toolBack
        visible: true
        anchors.bottom: parent.bottom
        iconId: "toolbar-back"; onClicked: { pageStack.pop(); }
    }

    ToolIcon {
        id: toolMenu
        visible: true
        platformIconId: "toolbar-view-menu"
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        onClicked: (imgMenu.status == DialogStatus.Closed) ? imgMenu.open() : imgMenu.close()
    }

    Menu {
        id: imgMenu
        visualParent: pageStack
        MenuLayout {
            MenuItem {
                text: qsTr("Start Slideshow")
                onClicked: {
                    onClicked: {
                        toolBack.visible = false;
                        toolMenu.visible = false;
                        imgTimer.running = true;
                        imgScreenSaver.setScreenSaverDelayed(true);
                    }
                }
            }
            MenuItem {
                text: qsTr("Open image in browser")
                onClicked: {
                    onClicked: { Qt.openUrlExternally(curModel.currentItem.myData.url)}
                }
            }
            MenuItem {
                text: qsTr("Read reddit comments ("+curModel.currentItem.myData.num_comments+")")
                onClicked: {
                    onClicked: {
                        Qt.openUrlExternally("http://www.reddit.com"+curModel.currentItem.myData.permalink)}
                }
            }
        }
    }
}
