/*
  Pinkit - Fun image viewer

  Copyright (C) 2012 Petteri Klemola <petteri@gmail.com>

  Pinkit is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 3 as
  published by the Free Software Foundation.

  Pinkit is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  02110-1301, USA.

  For more information about Pinkit visit http://23.fi/pinkit .

*/

import QtQuick 1.1
import com.nokia.meego 1.0
import "json2.js" as JSON
import "databasehandler.js" as DB

Page {
    id: thumbnailPage
    tools: commonTools

    property variant selectedThumbnail: ""
    property variant curModel: gridView
    property variant curMode: false

    Component.onCompleted: {
        theme.inverted = true
        console.log("loading imgs");
        curMode = DB.get_modes();
        load(0);
    }

    onStatusChanged: {
        if(status == PageStatus.Activating)
        {
            if (curMode != DB.get_modes())
            {
                curMode = DB.get_modes();
                console.log("mode change, reloading imgs");
                busyScreen.running = true;
                busyScreen.visible = true;
                thumbnailPage.load(0);
            }
        }
    }

    ListModel {
        id: view
    }

    function open_file(file)
    {
        var component = Qt.createComponent(file)
        if (component.status == Component.Ready)
        {
            pageStack.push(component);
        }
        else
        {
            console.log("Error loading component:", component.errorString());
        }
    }

    /* Return insert point to ListModel (view) */
    function bi_search(x)
    {
        var low = 0;
        var high = view.count - 1;
        while (low <= high)
        {
            var mid = Math.round((low + high) / 2);
            var midval = parseInt(view.get(mid).created);
            if (midval < x)
                low = mid + 1;
            else if (midval > x)
                high = mid -1;
            else
                return mid;
        }
        return low;
    }

    function loaded(x, index, reload)
    {
        var cur_index = index;
        for (var data in x["data"]["children"])
        {
            var url = "";
            var title = "";
            var thumbnail = "";
            var subreddit = "";
            var created = 0;
            var num_comments = 0;
            var permalink = "";

            for(var key in x["data"]["children"][data]["data"])
            {
                if (key == "url")
                {
                    url = x["data"]["children"][data]["data"][key];
                }
                else if (key == "thumbnail")
                {
                    thumbnail = x["data"]["children"][data]["data"][key];
                }
                else if (key == "title")
                {
                    title = x["data"]["children"][data]["data"][key];
                }
                else if (key == "subreddit")
                {
                    subreddit = x["data"]["children"][data]["data"][key];
                }
                else if (key == "created")
                {
                    /* Reverse the date, to get newest first in view */
                    created = (9999999999 - parseInt(x["data"]["children"][data]["data"][key]));
                }
                else if (key == "permalink")
                {
                    permalink = x["data"]["children"][data]["data"][key];
                }
                else if (key == "num_comments")
                {
                    num_comments = x["data"]["children"][data]["data"][key];
                }
            }

            console.log("reload ("+reload+") < created ("+created+")");

            if (reload > 0 && view.count > 0 &&
                reload <= created)
            {
                /* When reloading images add only new ones at the
                   top/start of the list */
                console.log("not new skippng");
                continue;
            }
            else
            {
                console.log("NEW image");
            }

            if (url && title && thumbnail &&
                thumbnail != "default" &&
                thumbnail != "nsfw")
            {
                var lower = url.toLowerCase();
                var ulen = url.length;
                var valid = false;

                /* Get proper image files */
                if (url[ulen-4] == ".")
                {
                    var supported = [".jpg", ".png", ".gif"];
                    for (var i in supported)
                    {
                        if (lower.substring(ulen-4) == supported[i])
                        {
                            valid = true;
                            break;
                        }
                    }
                }
                if (valid == false)
                {
                    var buff = "http://imgur.com/"
                    if (lower.substring(0,buff.length) == buff)
                    {
                        url = "http://i.imgur.com/"+url.substring(buff.length)+".jpg"
                        valid = true
                    }
                }
                if (valid)
                {
                    var i_index = 0;
                    if (view.count > 0)
                        i_index = bi_search(created);
                    view.insert(i_index,
                                {"url": url,
                                 "title": title,
                                 "thumbnail": thumbnail,
                                 "created" : created,
                                 "subreddit": subreddit,
                                 "permalink" : permalink,
                                 "num_comments" : num_comments });
                }
                else
                {
                    //console.log("Not a valid url: "+url);
                }
            }
            else
            {
                console.log("Not showing: "+url+" "+title+" "+thumbnail);
            }
        }
    }

    function get_json(urllist, index, reload)
    {
        var xhr = new XMLHttpRequest();
        var url = urllist[0]+"?limit="+DB.get_num_imgs();
        xhr.open("GET", url, true);
        urllist = urllist.splice(1);
        index++;
        xhr.onreadystatechange = function()
        {
            if (xhr.readyState == xhr.DONE)
            {
                if (xhr.status == 200)
                {
                    var jsonObject = JSON.parse(xhr.responseText);
                    loaded(jsonObject, index, reload);
                }
                else
                {
                    console.log("status "+url+" "+xhr.status);
                }
                if (urllist.length > 0)
                {
                    get_json(urllist, index, reload)
                }
                else if (reload > 0) /* last one and reload mode */
                {
                    busyReload.running = false;
                    busyReload.visible = false;
                    refreshIcon.visible = true;
                }
            }
        }
        xhr.send();
    }

    function load(reload)
    {
        var modes = 0;
        var urllist = null;

        if (reload == 0)
            view.clear();

        modes = DB.get_modes();

        if (DB.is_nsfw_mode(modes))
        {
            if (DB.is_gifs_mode(modes))
            {
                urllist = ["http://www.reddit.com/r/nsfw_gif.json"];
            }
            else
            {
                urllist = ["http://www.reddit.com/r/nsfw.json",
                           "http://www.reddit.com/r/amateur.json",
                           "http://www.reddit.com/r/AsianHotties.json",
                           "http://www.reddit.com/r/ass.json",
                           "http://www.reddit.com/r/redheads.json",
                           "http://www.reddit.com/r/boltedontits.json",
                           "http://www.reddit.com/r/sexybutnotporn.json",
                           "http://www.reddit.com/r/boobies.json",
                           "http://www.reddit.com/r/ginger.json",
                           "http://www.reddit.com/r/realgirls.json",
                           "http://www.reddit.com/r/xsmall.json",
                           "http://www.reddit.com/r/hotchickswithtattoos.json",
                           "http://www.reddit.com/r/nipples.json",
                           "http://www.reddit.com/r/nude.json",
                           "http://www.reddit.com/r/beach.json",
                           "http://www.reddit.com/r/upskirt.json",
                           "http://www.reddit.com/r/dyke.json",
                           "http://www.reddit.com/r/hipstergurlz.json"];
            }
        }
        else /* normal mode */
        {
            if (DB.is_gifs_mode(modes))
            {
                urllist = ["http://www.reddit.com/r/gifs.json"];
            }
            else
            {
                urllist = ["http://www.reddit.com/r/pics.json",
                           "http://www.reddit.com/r/images.json",
                           // "http://www.reddit.com/r/funny.json",
                           "http://www.reddit.com/r/art.json",
                           "http://www.reddit.com/r/itookapicture.json",
                           "http://www.reddit.com/r/picturechallenge.json",
                           "http://www.reddit.com/r/wallpaper.json",
                           "http://www.reddit.com/r/wallpapers.json",
                           "http://www.reddit.com/r/aww.json",
                           "http://www.reddit.com/r/aviation.json",
                           "http://www.reddit.com/r/birdpics.json",
                           "http://www.reddit.com/r/eyes.json", // slow update
                           "http://www.reddit.com/r/windowshots.json"];
            }
        }

        get_json(urllist, -1, reload);
    }

    Rectangle {
        id: centerPage
        color: "transparent"
        width: parent.width
        height: parent.height

        BusyIndicator {
            id: busyScreen
            platformStyle: BusyIndicatorStyle { size: "large" }
            anchors.verticalCenter: parent.verticalCenter
            anchors.horizontalCenter: parent.horizontalCenter
            visible: true
            running: true
        }
    }

    GridView {
        id: gridView
        anchors.fill: parent
        model: view
        cellWidth: 96; cellHeight: 96
        delegate: Column {
            property variant myData: model
            Rectangle {
                id: crec
                color: "black"
                width: 96
                height: 96

                Image {
                    id: image
                    width: parent.width
                    height: parent.height
                    fillMode: Image.PreserveAspectFit
                    smooth: true
                    source: thumbnail
                    onStatusChanged: {
                        if (image.status == Image.Ready)
                        {
                            busyScreen.running = false;
                            busyScreen.visible = false;
                        }
                    }
                }

                states : [
                    State {
                        name: "selected"
                        when: (thumbnail==selectedThumbnail)
                        PropertyChanges {
                            target: crec
                            width: 130
                            height: 130
                        }
                    }
                ]

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        gridView.currentIndex = index
                        selectedThumbnail = thumbnail;
                        onClicked: { thumbnailPage.open_file("ImgPage.qml")}
                    }
                }
            }
        }
        focus: true
    }

    ToolBarLayout {
        id: commonTools
        visible: true

        BusyIndicator {
            id: busyReload
            //platformStyle: BusyIndicatorStyle
            anchors.left: parent.left
            anchors.leftMargin: 20
            visible: false
            running: false
        }

        ToolIcon {
            id: refreshIcon
            platformIconId: "toolbar-refresh"
            visible: true
            anchors.left: parent.left
            onClicked: {
                //busyScreen.running = true;
                //busyScreen.visible = true;
                refreshIcon.visible = false;
                busyReload.running = true;
                busyReload.visible = true;
                thumbnailPage.load(parseInt(view.get(0).created));
            }
        }

        ToolIcon {
            platformIconId: "toolbar-view-menu"
            anchors.right: (parent === undefined) ? undefined : parent.right
            onClicked: (myMenu.status == DialogStatus.Closed) ? myMenu.open() : myMenu.close()
        }
    }

    Menu {
        id: myMenu
        visualParent: pageStack
        MenuLayout {
            MenuItem {
                text: qsTr("About")
                onClicked: {
                    onClicked: { thumbnailPage.open_file("AboutPage.qml")}
                }
            }
            MenuItem {
                text: qsTr("Settings")
                onClicked: {
                    onClicked: { thumbnailPage.open_file("SettingsPage.qml")}
                }
            }
        }
    }
}
